/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * main.c
 * Copyright (C) Maurice.Clerc@WriteMe.com  2012
	 *
 Let S(0,1) be the D-sphere centered on 0*, of radius 1.
	 Let x be a point on S
	 Let C be the cube around x, of side 2*delta
		or the sphere (x, delta)

	 We generate probeMax points inside C.
	 We count how many are also in the S.
	 The aim is to show that when delta>0 increases, the volume of the intersection
	 of S and C decreases (and tends to zero)

	 CubeSphereInter is free software: you can redistribute it and/or modify it
		 * under the terms of the GNU General Public License as published by the
		 * Free Software Foundation, either version 3 of the License, or
		 * (at your option) any later version.
			 * 
			 * CubeSphereInter is distributed in the hope that it will be useful, but
			 * WITHOUT ANY WARRANTY; without even the implied warranty of
			 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
			 * See the GNU General Public License for more details.
			 * 
			 * You should have received a copy of the GNU General Public License along
			 * with this program.  If not, see <http://www.gnu.org/licenses/>.
			 */

#include <stdio.h>
#include "math.h"
#include <stdlib.h>
#define DSup 10
struct vector 
{  
	int size;  
	double v[DSup]; 
};

double alea(double a, double b);
double alea_normal (double mean, double std_dev) ;
struct vector alea_sphere(int D, double radius);

FILE * f_run;

int main()
{
	int d;
	int DMax, DMin, D;
	double deltaMax, deltaMin, delta, deltaStep;
	int inside;
	double normy;
	int option;
	int probeMax, probeNb;
	struct vector V;
	double x[DSup]={0};
	double y[DSup];
	double zz;
	f_run = fopen ("f_run.txt", "w"); 

option=1; // 0 => Cube C inter Sphere S
			// 1 => Sphere C inter Sphere S
	DMin=2;
	DMax=10;
	x[0]=1; 
	deltaMax=2.5;
	deltaMin=0.1;
	deltaStep=0.1;
	probeMax=100000*DMax;

	delta=deltaMin;
	loopDelta:
		printf("\ndelta %f",delta);
		fprintf(f_run,"%f ",delta);
		for(D=DMin;D<=DMax;D++)
	{
		inside=0;
		for(probeNb=0;probeNb<probeMax;probeNb++)
		{
			normy=0;
			switch(option)
			{
				case 0: // Cube
					// Generate a point in a cube  around x
					// and check if it is also in the sphere (0,1)

					for(d=0;d<D;d++) 
				{
					y[d]=x[d]+ alea(-delta,delta);
					normy=normy+y[d]*y[d];
				}
				break;

				case 1: // Sphere
					V=alea_sphere (D,delta);
				for(d=0;d<D;d++) 
				{
					y[d]=V.v[d]+ x[d];
					normy=normy+y[d]*y[d];
				}
				break;
			}
			if(sqrt(normy)<1) inside++;
		}
		// Volume_intersection/Volume_C
		zz=(double)inside/probeMax;
		fprintf(f_run," %f",zz);
	}
		delta=delta+deltaStep;
		if(delta<=deltaMax) 
	{
		fprintf(f_run,"\n");
		goto loopDelta;
	}
		return (0);
}
// =============================================================================
double alea(double a, double b)
{
	double r;
	r=(double)rand()/RAND_MAX;
	return a+r*(b-a);
}
// =============================================================================
double alea_normal (double mean, double std_dev) 
{ 
	// Use the polar form of the Box-Muller transformation to obtain a pseudo
	// random number from a Gaussian distribution 

	double x1, x2, w, y1; 
	// WARNING. This method is valid only if alea (...) defines a (more or less)
	// uniform distribution. In particular, if "option" is so that the numbers are
	// read on a list, whose distribution is, say, Gaussian, then w will never be
	// smaller than 1 

	do  
	{
		x1 = 2.0 * alea (0, 1) - 1.0;
		x2 = 2.0 * alea (0, 1) - 1.0;
		w = x1 * x1 + x2 * x2; 
	}
	while (w >= 1.0); // WARNING. Infinite loop possible.  w may be _never_ < 1 !!


	w = sqrt (-2.0 * log (w) / w);
	y1 = x1 * w;

	if(alea(0,1)<0.5) y1=-y1; 
	y1 = y1 * std_dev + mean;
	return y1;  
}

// =============================================================================
struct vector alea_sphere(int D, double radius)
{
	/*  ******* Random point in a hypersphere ********
	 Maurice Clerc 2003-07-11
	 Last update: 2011-01-01
	 * 
	 Put  a random point inside the hypersphere S(center 0, radius 1), 
	 or on its surface
		 */

	int 	j;
	double   length;
	double pw;
	double      r;
	struct	vector	x;

	x.size=D;
	pw=1/(double)D;

	// ----------------------------------- Step 1.  Direction
	length=0;
	for (j=0;j<D;j++)
	{
		// Here a Gaussian distribution is needed
		x.v[j]=alea_normal(0,1);// Gaussian (Box-Muller method)
		length=length+  x.v[j]*x.v[j];
	}
	length=sqrt(length);
	//----------------------------------- Step 2. 

	r=alea(0,1); 
	for (j=0;j<D;j++)
	{
		x.v[j]=radius*r*x.v[j]/length;
	}
	return x;
}